SystemOrganization addCategory: #'System-Platforms'!

Object subclass: #OSPlatform
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'System-Platforms'!

!OSPlatform commentStamp: 'michael.rueger 2/25/2009 18:29' prior: 0!
An OSPlatform is an abstract representation of a 'OS platform'.
Platforms can be hierarchical, e.g., a "general" platform as superclass and more specific platforms as subclasses as long as the subclasses provide sufficient means to identify themselves.
The original implementation was for Tweak.

Current		holds the current OSPlatform subclass

Architectural considerations:
most platform specific methods that need to be added to the platform class should be in the form of extensions rather then adding them directly to this package. Otherwise the platform class will degenerate very quickly into a dependence hub for all kinds of sub systems.!

OSPlatform subclass: #MacOSXPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!MacOSXPlatform class methodsFor: 'private' stamp: 'StephaneDucasse 3/17/2010 21:31'!
isActivePlatform
	^Smalltalk isMacOS and: [Smalltalk osVersion asNumber >= 1000]! !

!MacOSXPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#MacOSX! !

!OSPlatform class methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:04'!
current
	"Answer the current platform"
	^Current! !

!OSPlatform class methodsFor: 'private' stamp: 'michael.rueger 2/25/2009 18:20'!
determineActivePlatformStartingAt: parentClass
	"Determine the current platform starting at parentClass.
	This is a potentially recursive process as we want to determine the most specific (in terms of subclasses) platform available."
	"OSPlatform determineActivePlatformStartingAt: OSPlatform"

	| platformClass |
	parentClass subclasses isEmpty
		ifTrue: [^parentClass].

	"Look for the matching platform class"
	platformClass := parentClass allSubclasses detect:[:any| any isActivePlatform] ifNone:[nil].

	"Check if there is a more specific subclass"
	^self determineActivePlatformStartingAt: platformClass! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'michael.rueger 3/2/2009 11:16'!
initialize
	"Initialize the receiver"
	"OSPlatform initialize"

	Smalltalk removeFromStartUpList: self.
	Smalltalk addToStartUpList: self after: Delay.
	Smalltalk removeFromShutDownList: self.
	Smalltalk addToShutDownList: self after: DisplayScreen.
	self startUp: true.! !

!OSPlatform class methodsFor: 'private' stamp: 'ar 12/11/2004 22:22'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^false! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:49'!
shutDown: quitting
	"The system is going down"
	Current ifNotNil:[Current shutDown: quitting].
! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'michael.rueger 2/25/2009 18:20'!
startUp: resuming
	"Determine the current platform.
	Use the most specific (in terms of subclasses) platform available."

	| platformClass |
	"Look for the matching platform class"
	platformClass := self determineActivePlatformStartingAt: self.
	platformClass
		ifNil: [^self].
	Current := platformClass new.
	Current startUp: resuming! !

!OSPlatform class methodsFor: 'accessing' stamp: 'michael.rueger 2/27/2009 17:25'!
virtualKey: virtualKeyCode
	"Subclass responsibility to override if necessary"
	^nil! !

!OSPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	"Returns a symbol specific to the platform family (MacOSX, Windows, Unix, RiscOS).
	This may need to be extended if there are new platforms added or significant differences within a platform family arise (as was the case between MacOS 9 and X)."

	self subclassResponsibility! !

!OSPlatform methodsFor: 'initialize' stamp: 'ar 12/11/2004 22:47'!
shutDown: quitting
	"Squeak is shutting down. If this platform requires specific shutdown code, this is a great place to put it."
! !

!OSPlatform methodsFor: 'initialize' stamp: 'michael.rueger 2/25/2009 18:11'!
startUp: resuming
	"Squeak is starting up. If this platform requires specific intialization, this is a great place to put it."
! !

!OSPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 22:19'!
virtualKey: virtualKeyCode
	^self class virtualKey: virtualKeyCode! !

OSPlatform subclass: #RiscOSPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!RiscOSPlatform classSide methodsFor: 'private' stamp: 'StephaneDucasse 3/18/2010 21:22'!
isActivePlatform
	^Smalltalk isRiscOS ! !

!RiscOSPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#RiscOS! !

OSPlatform subclass: #UnixPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!UnixPlatform class methodsFor: 'private' stamp: 'StephaneDucasse 3/18/2010 21:50'!
isActivePlatform
	^Smalltalk isUnix! !

!UnixPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#Unix! !

OSPlatform subclass: #Win32Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!Win32Platform class methodsFor: 'private' stamp: 'StephaneDucasse 3/17/2010 21:25'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^Smalltalk isWin32! !

!Win32Platform class methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 11:20'!
virtualKey: virtualKeyCode
	"Win32Platform virtualKey: $C charCode"

	(virtualKeyCode <=  90 "$Z charCode"
		and: [virtualKeyCode >=  65 "$A "])
		ifFalse: [^nil].
	"#($a $b $c $d $e $f $g $h $i $j $k $l $m $n $o $p $q $r $s $t $u $v $w $x $y $z)"
	
	^(#($a nil $c $d nil $f $g nil nil nil nil $l $m $n nil $p nil nil $s nil nil $v nil $x nil $z)
		at: virtualKeyCode-64) ifNotNil: [:char | char charCode]! !

!Win32Platform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#Windows! !
OSPlatform initialize!
